/* ---------------------------------------------------------------------
 * 
 * Giada - Your Hardcore Loopmachine 
 * 
 * ---------------------------------------------------------------------
 * 
 * Copyright (C) 2010-2012 Giovanni A. Zuliani | Monocasual
 * 
 * This file is part of Giada - Your Hardcore Loopmachine.
 * 
 * Giada - Your Hardcore Loopmachine is free software: you can 
 * redistribute it and/or modify it under the terms of the GNU General 
 * Public License as published by the Free Software Foundation, either 
 * version 3 of the License, or (at your option) any later version.
 * 
 * Giada - Your Hardcore Loopmachine is distributed in the hope that it 
 * will be useful, but WITHOUT ANY WARRANTY; without even the implied 
 * warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Giada - Your Hardcore Loopmachine. If not, see 
 * <http://www.gnu.org/licenses/>.
 * 
 * ------------------------------------------------------------------ */




#include "patch.h"


extern Recorder G_Recorder;
extern Mixer 		G_Mixer;
extern Conf 		G_Conf;


int Patch::open(const char *file) {
	fp = fopen(file, "r");
	if (fp == NULL)					
		return PATCH_UNREADABLE;

	if (getValue("header") != "GIADAPTC")
		return PATCH_INVALID;

	return PATCH_OPEN_OK;
}





void Patch::setDefault() {
	name[0]    = '\0';
  lastTakeId = 0;
  samplerate = DEFAULT_SAMPLERATE;
}





int Patch::close() {
	return fclose(fp);
}





void Patch::getName() {
	std::string out = getValue("patchname");
	strncpy(name, out.c_str(), MAX_PATCHNAME_LEN);
}





std::string Patch::getSamplePath(int c) {
	char tmp[16];
	sprintf(tmp, "samplepath%d", c);

	
	return getValue(tmp);
}





float Patch::getPitch(int c) {
	char tmp[16];
	sprintf(tmp, "chanPitch%d", c);
	float out = atof(getValue(tmp).c_str());
	if (out > 2.0f || out < 0.1f)
		return 1.0f;
	return out;
}





float Patch::getVol(int c) {
	char tmp[16];
	sprintf(tmp, "chanvol%d", c);
	float out = atof(getValue(tmp).c_str());
	if (out > 1.0f || out < 0.0f)
		return DEFAULT_VOL;
	return out;
}





int Patch::getMode(int c) {
	char tmp[16];
	sprintf(tmp, "chanmode%d", c);
	int out = atoi(getValue(tmp).c_str());
	if (out & (LOOP_BASIC | LOOP_ONCE | LOOP_REPEAT | SINGLE_BASIC | SINGLE_PRESS | SINGLE_RETRIG))
		return out;
	return DEFAULT_CHANMODE;
}





int Patch::getMute(int c) {
	char tmp[16];
	sprintf(tmp, "chanmute%d", c);
	return atoi(getValue(tmp).c_str());
}





int Patch::getStart(int c) {
	char tmp[16];
	sprintf(tmp, "chanstart%d", c);
	int out = atoi(getValue(tmp).c_str());
	if (out < 0)
		return 0;
	return out;
}





int Patch::getEnd(int c, unsigned size) {
	char tmp[16];
	sprintf(tmp, "chanend%d", c);

	
	std::string val = getValue(tmp);
	if (val == "")
		return size;

	unsigned out = atoi(val.c_str());
	if (out < 0 || out > size)
		return size;
	return out;
}





float Patch::getBoost(int c) {
	char tmp[16];
	sprintf(tmp, "chanBoost%d", c);
	float out = atof(getValue(tmp).c_str());
	if (out < 1.0f || out > 20.0f)
		return DEFAULT_BOOST;
	return out;
}





float Patch::getPanLeft(int c) {
	char tmp[16];
	sprintf(tmp, "chanPanLeft%d", c);

	
	std::string val = getValue(tmp);
	if (val == "")
		return 1.0f;

	float out = atof(val.c_str());
	if (out < 0.0f || out > 1.0f)
		return 1.0f;
	return out;
}





float Patch::getPanRight(int c) {
	char tmp[16];
	sprintf(tmp, "chanPanRight%d", c);

	
	std::string val = getValue(tmp);
	if (val == "")
		return 1.0f;

	float out = atof(val.c_str());
	if (out < 0.0f || out > 1.0f)
		return 1.0f;
	return out;
}





bool Patch::getRecActive(int c) {
	char tmp[16];
	sprintf(tmp, "chanRecActive%d", c);
	return atoi(getValue(tmp).c_str());
}





float Patch::getOutVol() {
	return atof(getValue("outVol").c_str());
}





float Patch::getInVol() {
	return atof(getValue("inVol").c_str());
}





float Patch::getBpm() {
	float out = atof(getValue("bpm").c_str());
	if (out < 20.0f || out > 999.0f)
		return DEFAULT_BPM;
	return out;
}





int Patch::getBars() {
	int out = atoi(getValue("bars").c_str());
	if (out <= 0 || out > 32)
		return DEFAULT_BARS;
	return out;
}





int Patch::getBeats() {
	int out = atoi(getValue("beats").c_str());
	if (out <= 0 || out > 32)
		return DEFAULT_BEATS;
	return out;
}





int Patch::getQuantize() {
	int out = atoi(getValue("quantize").c_str());
	if (out < 0 || out > 8)
		return DEFAULT_QUANTIZE;
	return out;
}





bool Patch::getMetronome() {
	bool out = atoi(getValue("metronome").c_str());
	if (out != true || out != false)
		return false;
	return out;
}





int Patch::getLastTakeId() {
	
	return atoi(getValue("lastTakeId").c_str());
}





int Patch::getSamplerate() {
	int out = atoi(getValue("samplerate").c_str());
	if (out <= 0)
		return DEFAULT_SAMPLERATE;
	return out;
}





int Patch::readRecs(const char *file) {
	fp = fopen(file, "r");
	if (fp == NULL)
		return 0;

	printf("[PATCH] Reading recs...\n");

	unsigned numrecs = atoi(getValue("numrecs").c_str());

	for (unsigned i=0; i<numrecs; i++) {
		int frame, recPerFrame;

		
		char tmpbuf[16];
		sprintf(tmpbuf, "recframe%d", i);
		sscanf(getValue(tmpbuf).c_str(), "%d %d", &frame, &recPerFrame);



		for (int k=0; k<recPerFrame; k++) {
			int chan, type;

			
			char tmpbuf[16];
			sprintf(tmpbuf, "f%da%d", i, k);
			sscanf(getValue(tmpbuf).c_str(), "%d|%d", &chan, &type);



			
			if (frame % 2 != 0)
				frame++;

			G_Recorder.rec(chan, type, frame);
		}
	}

	fclose(fp);
	return 1;
}





int Patch::write(const char *file, const char *name) {
	fp = fopen(file, "w");
	if (fp == NULL)
		return 0;

	fprintf(fp, "# --- Giada patch file --- \n");
	fprintf(fp, "header=GIADAPTC\n");
	fprintf(fp, "version=%s\n", VERSIONE);
	fprintf(fp, "patchname=%s\n", name);

	for (unsigned i=0; i<MAX_NUM_CHAN; i++) {
		fprintf(fp, "samplepath%d=%s\n",    i, G_Mixer.chan[i] == NULL ? "" : G_Mixer.chan[i]->pathfile.c_str());
		fprintf(fp, "chanmute%d=%d\n",      i, G_Mixer.chanMute[i]);
		fprintf(fp, "chanvol%d=%f\n",       i, G_Mixer.chanVolume[i]);
		fprintf(fp, "chanmode%d=%d\n",      i, G_Mixer.chanMode[i]);
		fprintf(fp, "chanstart%d=%d\n",     i, G_Mixer.chanStartTrue[i]);  
		fprintf(fp, "chanend%d=%d\n",       i, G_Mixer.chanEndTrue[i]);    
		fprintf(fp, "chanBoost%d=%f\n",     i, G_Mixer.chanBoost[i]);
		fprintf(fp, "chanPanLeft%d=%f\n",   i, G_Mixer.chanPanLeft[i]);
		fprintf(fp, "chanPanRight%d=%f\n",  i, G_Mixer.chanPanRight[i]);
		fprintf(fp, "chanRecActive%d=%d\n", i, G_Recorder.chanActive[i]);
		fprintf(fp, "chanPitch%d=%f\n",     i, G_Mixer.chanPitch[i]);
	}

	fprintf(fp, "bpm=%f\n",        G_Mixer.bpm);
	fprintf(fp, "bars=%d\n",       G_Mixer.bars);
	fprintf(fp, "beats=%d\n",      G_Mixer.beats);
	fprintf(fp, "quantize=%d\n",   G_Mixer.quantize);
	fprintf(fp, "outVol=%f\n",     G_Mixer.outVol);
	fprintf(fp, "inVol=%f\n",      G_Mixer.inVol);
	fprintf(fp, "metronome=%d\n",  G_Mixer.metronome);
	fprintf(fp, "lastTakeId=%d\n", lastTakeId);
	fprintf(fp, "samplerate=%d\n", G_Conf.samplerate);	

	
	fprintf(fp, "numrecs=%d\n", G_Recorder.global.size);
	for (unsigned i=0; i<G_Recorder.global.size; i++) {
		fprintf(fp, "recframe%d=%d %d\n", i, G_Recorder.frames.at(i), G_Recorder.global.at(i).size);
		for (unsigned k=0; k<G_Recorder.global.at(i).size; k++) {
			fprintf(fp, "f%da%d=%d|%d\n",
				i, k,
				G_Recorder.global.at(i).at(k)->chan,
				G_Recorder.global.at(i).at(k)->type);
		}
	}

	fclose(fp);
	return 1;
}
